const TelegramBot = require("node-telegram-bot-api");
const mysql = require("mysql2");
const bot = new TelegramBot(process.env.TELEGRAM_TOKEN, { polling: false });
const moment = require("moment");
const adminGestioneMissioni = require("./AdminGestioneMissioni.js");

const db = mysql.createConnection({
  host: process.env.DB_HOST,
  user: process.env.DB_USER,
  password: process.env.DB_PASSWORD,
  database: process.env.DB_NAME,
});

const handleBotMessage = async (userMessages, chatId, text, options) => {
  try {
    let sentMessage;

    // Check if we're sending a photo or text
    if (options && options.photo) {
      // Send a photo
      sentMessage = await bot.sendPhoto(chatId, options.photo, {
        caption: text,
      });
    } else {
      // Send a text message
      sentMessage = await bot.sendMessage(chatId, text, options);
    }

    if (!userMessages[chatId]) {
      userMessages[chatId] = [];
    }

    userMessages[chatId].push(sentMessage.message_id);
  } catch (err) {
    console.error("Errore nell'invio del messaggio:", err);
  }
};

const paymentStats = {};

const addPointInvitedById = (userMessages, mission, invitedById, points) => {
  // Recupera il ruolo dell'utente invitato e il telegram_id per determinare la percentuale e inviare il messaggio
  const userQuery = `SELECT role, wallet_friends, telegram_id FROM telegram_users WHERE id = ?`;

  db.query(userQuery, [invitedById], (err, result) => {
    if (err) {
      console.error("Errore nel recupero dell'utente invitato:", err);
      return;
    }

    const userRole = result[0].role;
    let walletFriends = parseFloat(result[0].wallet_friends);
    const telegram_id = result[0].telegram_id;
    let percentage = 0;

    // Determina la percentuale in base al tipo di missione e al ruolo dell'utente
    if (mission.type === "normal") {
      // Missioni normali
      if (userRole === "user") {
        percentage = 0.2; // 20% per utenti base
      } else if (userRole === "user_vip") {
        percentage = 0.35; // 35% per utenti premium
      }
    } else {
      // Altre missioni
      if (userRole === "user") {
        percentage = 0.05; // 5% per utenti base
      } else if (userRole === "user_vip") {
        percentage = 0.1; // 10% per utenti premium
      }
    }

    // Calcola i punti da aggiungere in base alla percentuale
    const pointsToAdd = parseFloat((points * percentage).toFixed(2));

    // Aggiungi i punti al wallet_friends dell'utente invitato
    walletFriends += pointsToAdd;

    // Rounding walletFriends before update
    walletFriends = parseFloat(walletFriends.toFixed(2));

    const updateWalletQuery = `
      UPDATE telegram_users
      SET wallet_friends = ?
      WHERE id = ?`;

    db.query(updateWalletQuery, [walletFriends, invitedById], (err, result) => {
      if (err) {
        console.error(
          "Errore nell'aggiornamento del wallet dell'utente invitato:",
          err
        );
      } else {
        console.log(
          `Aggiunti ${pointsToAdd} punti al wallet_friends dell'utente con ID ${invitedById}.`
        );

        // Invia messaggio all'utente invitante tramite handleBotMessage
        handleBotMessage(
          userMessages,
          telegram_id,
          `🥳 Complimenti! Un utente che hai invitato ha completato una missione! Hai guadagnato ${pointsToAdd} punti!`
        );
      }
    });
  });
};

const getMissionByRevisione = (id) => {
  return new Promise((resolve, reject) => {
    const query = `
      SELECT m.*, tu.telegram_id, tu.wallet_missions, tu.invitedById
      FROM mission_status as ms
      INNER JOIN missions as m ON m.id = ms.mission_id
      INNER JOIN telegram_users as tu ON tu.id = ms.user_id
      WHERE ms.id = ?
      LIMIT 1`;
    db.query(query, [id], (err, results) => {
      if (err) {
        console.error("Errore nel recupero dell'utente:", err);
        reject(err);
      } else {
        resolve(results[0]);
      }
    });
  });
};

const incrementMissionCountUser = (missionId) => {
  // Prima recuperiamo i valori attuali di count_user e max_user per la missione
  const missionQuery = `
    SELECT count_user, max_user, status
    FROM missions
    WHERE id = ?`;

  db.query(missionQuery, [missionId], (err, result) => {
    if (err) {
      console.error("Errore nel recupero della missione:", err);
      return;
    }

    const currentCountUser = result[0].count_user;
    const maxUser = result[0].max_user;
    const currentStatus = result[0].status;

    // Se la missione è già inattiva, non fare nulla
    if (currentStatus === "inactive") {
      console.log(`La missione con ID ${missionId} è già inattiva.`);
      return;
    }

    // Incrementa count_user
    const newCountUser = currentCountUser + 1;

    // Se il nuovo count_user è >= max_user, imposta la missione su "inactive"
    let newStatus = currentStatus;
    if (newCountUser >= maxUser) {
      newStatus = "inactive";
    }

    // Aggiorna count_user e, se necessario, lo stato della missione
    const updateQuery = `
      UPDATE missions
      SET count_user = ?, status = ?
      WHERE id = ?`;

    db.query(
      updateQuery,
      [newCountUser, newStatus, missionId],
      (err, result) => {
        if (err) {
          console.error("Errore nell'aggiornamento della missione:", err);
        } else {
          console.log(
            `La missione con ID ${missionId} è stata aggiornata. count_user: ${newCountUser}, status: ${newStatus}`
          );
        }
      }
    );
  });
};

const setStatusMission = (userMessages, id, status) => {
  const query = `
    UPDATE mission_status
    SET status = ? WHERE id = ?`;

  db.query(query, [status, id], async (err, result) => {
    if (err) {
      console.error(
        "Errore nell'aggiornamento dello stato della revisione:",
        err
      );
    } else {
      console.log(`Stato della revisione con id ${id} aggiornato a ${status}`);

      if (status === "complete") {
        const mission = await getMissionByRevisione(id);

        if (mission.type !== "advertising") {
          const points = mission.points;
          let updatedPoints = points;
          const invitedById = mission.invitedById;

          // Recupera il ruolo dell'utente dal mission.telegram_id
          const userRoleQuery = `
            SELECT role, wallet_missions FROM telegram_users WHERE telegram_id = ?`;
          db.query(userRoleQuery, [mission.telegram_id], (err, result) => {
            if (err) {
              console.error("Errore nel recupero del ruolo dell'utente:", err);
            } else {
              const userRole = result[0].role;
              let newWalletMissions = parseFloat(result[0].wallet_missions);
              let updatedPoints = parseFloat(points);

              // Se l'utente è user_vip, aggiungi il 10% ai punti
              if (userRole === "user_vip") {
                if (mission.type === "normal") {
                  updatedPoints += updatedPoints * 0.5;
                } else {
                  updatedPoints += updatedPoints * 0.1;
                }
              }

              // Aggiorna il wallet_missions dell'utente
              newWalletMissions += updatedPoints;

              // Round to 2 decimal places for currency
              newWalletMissions = parseFloat(newWalletMissions.toFixed(2));

              const updateWalletQuery = `
                UPDATE telegram_users
                SET wallet_missions = ?
                WHERE telegram_id = ?`;
              db.query(
                updateWalletQuery,
                [newWalletMissions, mission.telegram_id],
                (err, result) => {
                  if (err) {
                    console.error("Errore nell'aggiornamento del wallet:", err);
                  } else {
                    if (mission.type === "close_number") {
                      incrementMissionCountUser(mission.id);
                    }

                    handleBotMessage(
                      userMessages,
                      mission.telegram_id,
                      `🥳 Complimenti! Hai completato la sfida con ID ${id} e hai guadagnato ${updatedPoints} punti!`
                    );
                    if (invitedById) {
                      addPointInvitedById(
                        userMessages,
                        mission,
                        invitedById,
                        points
                      );
                    }
                  }
                }
              );
            }
          });

          // Se non è admin, sottrai i punti al creatore della missione
          if (!mission.user_admin) {
            const creatorQuery = `
              SELECT wallet_missions, telegram_id FROM telegram_users WHERE id = ?`;
            db.query(creatorQuery, [mission.user_id], (err, result) => {
              if (err) {
                console.error(
                  "Errore nel recupero del wallet dell'utente creatore:",
                  err
                );
              } else {
                let creatorWallet = result[0].wallet_missions;
                let creatorChatId = result[0].telegram_id;

                // Sottrai i punti dal wallet del creatore della missione
                creatorWallet -= points;

                // Aggiorna il wallet del creatore della missione
                const updateCreatorWalletQuery = `
                  UPDATE telegram_users
                  SET wallet_missions = ?
                  WHERE id = ?`;
                db.query(
                  updateCreatorWalletQuery,
                  [creatorWallet, mission.user_id],
                  (err, result) => {
                    if (err) {
                      console.error(
                        "Errore nell'aggiornamento del wallet del creatore:",
                        err
                      );
                    } else {
                      handleBotMessage(
                        userMessages,
                        creatorChatId,
                        `🥳 Complimenti! Un utente ha completato la tua missione, i tuoi punti sono stati scalati!`
                      );

                      // Se il creatore non ha abbastanza punti, imposta la missione su "inactive"
                      if (creatorWallet < points) {
                        const updateMissionStatusQuery = `
                        UPDATE mission_status
                        SET status = 'inactive'
                        WHERE id = ?`;
                        db.query(
                          updateMissionStatusQuery,
                          [id],
                          (err, result) => {
                            if (err) {
                              console.error(
                                "Errore nell'aggiornamento dello stato della missione:",
                                err
                              );
                            } else {
                              handleBotMessage(
                                userMessages,
                                creatorChatId,
                                `😞 Missione con ID ${id} impostata su inactive a causa di punti insufficienti.!`
                              );

                              console.log(
                                `Missione con ID ${id} impostata su inactive a causa di punti insufficienti.`
                              );
                            }
                          }
                        );
                      }
                    }
                  }
                );
              }
            });
          }
        }
      }

      if (status === "rejected") {
        const mission = await getMissionByRevisione(id);

        handleBotMessage(
          userMessages,
          mission.telegram_id,
          `😞 Mi Spiace! La sfida con ID ${id} è stata rifiutata.`
        );
      }
    }
  });
};

const getRevisione = (page = 1, pageSize = 1) => {
  return new Promise((resolve, reject) => {
    const offset = (page - 1) * pageSize;
    const query = `
      SELECT ms.*, m.link, m.points, m.platform, m.action_type, tu.telegram_id, tu.wallet_missions, tu.instagram, tu.tiktok, tu.facebook
      FROM mission_status as ms
      INNER JOIN missions as m ON m.id = ms.mission_id
      INNER JOIN telegram_users as tu ON tu.id = ms.user_id
      WHERE ms.status = 'under_review'
      LIMIT ?, ?`; // Usa LIMIT con offset e pageSize
    db.query(query, [offset, pageSize], (err, results) => {
      if (err) {
        console.error("Errore nel recupero dell'utente:", err);
        reject(err);
      } else {
        resolve(results);
      }
    });
  });
};

const sendRevisione = async (chatId, userMessages, page = 1) => {
  const revisioni = await getRevisione(page);
  const revisione = revisioni[0];

  if (!revisione) {
    await handleBotMessage(
      userMessages,
      chatId,
      `Nessuna richiesta in attesa`,
      startKeyboard
    );
  } else {
    let keyboardRev = [
      [
        { text: "✅ Convalida", callback_data: "con-rev-" + revisione.id },
        {
          text: "❌ Rifiuta Richiesta",
          callback_data: "rif-rev-" + revisione.id,
        },
      ],
      [
        {
          text: "➡️ Prossima Richiesta",
          callback_data: "rev-" + (page + 1),
        },
      ],

      [{ text: "↩️ Torna indietro", callback_data: "annulla" }],
    ];

    if (revisione.attached) {
      keyboardRev.splice(1, 0, [
        {
          text: "🖼 Visualizza Screenshot",
          callback_data: "see-screen-" + page,
        },
      ]);
    }
    const options = {
      disable_web_page_preview: true,
      parse_mode: "HTML",
      reply_markup: {
        inline_keyboard: keyboardRev,
      },
    };

    await handleBotMessage(
      userMessages,
      chatId,
      `ID Richiesta: ${revisione.id}
ID Missione: ${revisione.mission_id}

Piattaforma: ${revisione.platform}
Link: ${revisione.link}
Azione: ${revisione.action_type}

Crediti: ${revisione.points}
Testo: ${revisione.text_added ?? "No"}

Account: ${
        revisione.platform === "telegram"
          ? revisione.telegram_id
          : revisione[revisione.platform]
      }`,
      options
    );
  }
};
const startKeyboard = {
  parse_mode: "HTML",
  reply_markup: {
    keyboard: [
      [{ text: "🚀 Missioni 🚀" }, { text: "📜 Pubblicità 📜" }],
      [{ text: "👀 Revisione 👀" }, { text: "💲 Pagamenti 💲" }],
      [{ text: "🟡 Missioni in Approvazione 🟡" }],
      [{ text: "↩️ Torna indietro" }],
    ],
    resize_keyboard: true,
  },
};

const conPromise = db.promise();

async function getSinglePayment(page = 1) {
  const offset = page - 1;
  const query = `SELECT transactions.*, telegram_users.telegram_id as chatId  FROM transactions LEFT JOIN telegram_users ON transactions.userId = telegram_users.id  WHERE status = 'in_progress' LIMIT 1 OFFSET ?`;
  const [rows] = await conPromise.query(query, [offset]);
  return rows[0]; // Return the first payment
}

function generatePaymentKeyboard(payment, page, totalPages) {
  const inlineKeyboard = [
    [
      {
        text: "✅ Approva",
        callback_data: `p-appr-${payment.id}-${payment.chatId}`,
      },
      {
        text: "❌ Rifiuta",
        callback_data: `p-reject-${payment.id}-${payment.chatId}`,
      },
    ],
  ];

  // Add pagination controls
  const paginationRow = [];
  if (page > 1) {
    paginationRow.push({
      text: "⬅️ Indietro",
      callback_data: `p-page-${page - 1}`,
    });
  }
  if (page < totalPages) {
    paginationRow.push({
      text: "➡️ Avanti",
      callback_data: `p-page-${page + 1}`,
    });
  }
  if (paginationRow.length > 0) {
    inlineKeyboard.push(paginationRow);
  }

  inlineKeyboard.push([
    { text: "↩️ Torna indietro", callback_data: "annulla" },
  ]);

  return inlineKeyboard;
}

async function displaySinglePayment(userMessages, chatId, page = 1) {
  const totalPayments = await conPromise.query(
    "SELECT COUNT(*) AS count FROM transactions WHERE status = 'in_progress'"
  );
  const totalPages = Math.ceil(totalPayments[0][0].count / 1); // 1 payment per page

  const payment = await getSinglePayment(page);

  if (payment) {
    const formattedDate = moment(payment.date).format("DD-MM-YYYY");
    const text =
      `ID Richiesta: ${payment.uuid}\n` +
      `Metodo: ${payment.type}\n` +
      `Utente Richiedente: ${payment.chatId}\n` +
      `${payment.wallet ? "Indirizzo: " + payment.wallet : ""}\n` +
      `Importo: ${payment.amount} €\n` +
      `Data: ${formattedDate}`;

    const options = {
      parse_mode: "HTML",
      reply_markup: {
        inline_keyboard: generatePaymentKeyboard(payment, page, totalPages),
        resize_keyboard: true,
      },
    };

    await handleBotMessage(userMessages, chatId, text, options);
  } else {
    await handleBotMessage(
      userMessages,
      chatId,
      "Nessun pagamento in corso.",
      startKeyboard
    );
  }
}

// Handle approve (insert transaction_code)
async function handleApprovePayment(paymentId, transactionCode) {
  const query = `UPDATE transactions SET transaction_code = ?, status = 'paid' WHERE id = ?`;
  await conPromise.query(query, [transactionCode, paymentId]);
}

// Handle reject (insert reason)
async function handleRejectPayment(paymentId, reason) {
  try {
    // Retrieve the amount and user_id from the transaction
    const [transaction] = await conPromise.query(
      `SELECT amount, userId FROM transactions WHERE id = ?`,
      [paymentId]
    );

    if (transaction.length === 0) {
      throw new Error("Transaction not found");
    }

    const { amount, userId } = transaction[0];

    // Calculate the value to add to the wallet
    const walletBonus = amount / 0.001;

    // Update the transactions table with reason and status
    const updateTransactionQuery = `UPDATE transactions SET reason = ?, status = 'cancelled' WHERE id = ?`;
    await conPromise.query(updateTransactionQuery, [reason, paymentId]);

    // Update the user's wallet
    const updateWalletQuery = `UPDATE telegram_users SET wallet_missions = wallet_missions + ? WHERE id = ?`;
    await conPromise.query(updateWalletQuery, [walletBonus, userId]);
  } catch (error) {
    console.error("Error handling payment rejection:", error);
    // Handle error accordingly
  }
}

async function getSinglePendingMission(page = 1) {
  const offset = page - 1;
  const query = `SELECT * FROM missions WHERE status = 'pending' LIMIT 1 OFFSET ?`;
  const [rows] = await conPromise.query(query, [offset]); // Use conPromise.query()
  return rows[0];
}

// Generate inline keyboard with Approve and Delete buttons for one mission
function generateSingleMissionKeyboard(mission, page, totalPages) {
  const inlineKeyboard = [
    [
      { text: "✅ Approva", callback_data: `m-appr-${mission.id}` },
      { text: "❌ Non approva", callback_data: `m-delete-${mission.id}` },
    ],
  ];

  // Add pagination controls
  const paginationRow = [];
  if (page > 1) {
    paginationRow.push({
      text: "⬅️ Indietro",
      callback_data: `m-page-${page - 1}`,
    });
  }
  if (page < totalPages) {
    paginationRow.push({
      text: "➡️ Avanti",
      callback_data: `m-page-${page + 1}`,
    });
  }
  if (paginationRow.length > 0) {
    inlineKeyboard.push(paginationRow);
  }

  inlineKeyboard.push([
    { text: "↩️ Torna indietro", callback_data: "annulla" },
  ]);

  return inlineKeyboard;
}

// Handle displaying a single pending mission
async function displaySinglePendingMission(userMessages, chatId, page = 1) {
  const totalMissions = await conPromise.query(
    "SELECT COUNT(*) AS count FROM missions WHERE status = 'pending'"
  );
  const totalPages = Math.ceil(totalMissions[0][0].count / 1);

  const mission = await getSinglePendingMission(page);

  if (mission) {
    const text =
      `⭐️ Missione Social: ${capitalizeFirstLetter(mission.platform)} ⭐️\n` +
      `👉🏻 ${mission.action_type} un utente\n` +
      `💰 Guadagnerai: ${mission.points} punti.\n` +
      `📍 Link: ${mission.link}\n`;

    const options = {
      parse_mode: "HTML",
      reply_markup: {
        inline_keyboard: generateSingleMissionKeyboard(
          mission,
          page,
          totalPages
        ),
        resize_keyboard: true,
      },
    };

    await handleBotMessage(userMessages, chatId, text, options);
  } else {
    await handleBotMessage(
      userMessages,
      chatId,
      "Nessuna missione in approvazione.",
      startKeyboard
    );
  }
}

function capitalizeFirstLetter(string) {
  return string.charAt(0).toUpperCase() + string.slice(1);
}

const handleAdmin = async (userMessages, chatId, userData, msg) => {
  adminGestioneMissioni.handleGestioneMissioni(
    userMessages,
    chatId,
    userData,
    msg
  );

  if (msg.text === "/start" || msg.text === "↩️ Torna indietro") {
    paymentStats[chatId] = null;
  }

  if (msg.text === "/admin" || msg.text === "🎓 Admin 🎓") {
    await handleBotMessage(userMessages, chatId, `Zona admin`, startKeyboard);
  }

  if (msg.text.startsWith("/rispondi")) {
    // Esempio: /rispondi 148141220 ciao ciao ciao
    const parts = msg.text.split(" ");

    if (parts.length >= 3) {
      const targetChatId = parts[1];
      const responseMessage = parts.slice(2).join(" ");

      // Assicurati che il targetChatId sia un numero
      if (!isNaN(targetChatId)) {
        await bot.sendMessage(
          targetChatId,
          "<i>Utilizza /admin &lt;messaggio&gt; per rispondere</i>", // Usa &lt; e &gt; per rappresentare < e >
          { parse_mode: "HTML" }
        );
        await bot.sendMessage(targetChatId, "Admin: " + responseMessage);

        await bot.sendMessage(
          chatId,
          `Messaggio inviato a ${targetChatId}: ${responseMessage}`,
          startKeyboard
        );
      } else {
        await bot.sendMessage(chatId, `ID chat non valido: ${targetChatId}`);
      }
    } else {
      await bot.sendMessage(
        chatId,
        `Formato comando non valido. Usa: /rispondi <chatId> <messaggio>`
      );
    }
  }

  if (msg.text === "👀 Revisione 👀") {
    sendRevisione(chatId, userMessages, 1);
  }

  if (msg.text === "🟡 Missioni in Approvazione 🟡") {
    await displaySinglePendingMission(userMessages, chatId, 1);
  }

  if (msg.text === "💲 Pagamenti 💲") {
    await displaySinglePayment(userMessages, chatId, 1);
  }

  if (paymentStats[chatId]) {
    if (paymentStats[chatId]?.type === "appr") {
      await handleApprovePayment(paymentStats[chatId].transactionId, msg.text);

      await displaySinglePayment(userMessages, chatId);

      await bot.sendMessage(
        paymentStats[chatId].userPay,
        `💰 Complimenti! La tua richiesta di prelievo con ID ${paymentStats[chatId].transactionId} è stata accettata. In base al metodo di prelievo scelto, riceverai la somma prelevata in 24/48 ore.`
      );
      paymentStats[chatId] = null;
    }

    if (paymentStats[chatId]?.type === "reject") {
      await handleRejectPayment(paymentStats[chatId].transactionId, msg.text);

      await displaySinglePayment(userMessages, chatId);

      await bot.sendMessage(
        paymentStats[chatId].userPay,
        `😞 Mi Spiace! La tua richiesta di prelievo con ID ${paymentStats[chatId].transactionId} non è stata accettata. \nMotivo: ${msg.text}`
      );
      paymentStats[chatId] = null;
    }
  }
};

async function handleApproveMission(missionId) {
  const query = `UPDATE missions SET approved = true, status = 'active' WHERE id = ?`;
  await conPromise.query(query, [missionId]);
}

// Delete mission handler
async function handleDeleteMission(missionId) {
  const query = `DELETE FROM missions WHERE id = ?`;
  await conPromise.query(query, [missionId]);
}

const handleCallbackQuery = async (userMessages, chatId, userData, msg) => {
  adminGestioneMissioni.handleCallbackQuery(
    userMessages,
    chatId,
    userData,
    msg
  );
  if (msg.data === "annulla") {
    paymentStats[chatId] = null; // Resetta lo stato dell'utente
  }

  if (msg.data.startsWith("see-screen-")) {
    const page = msg.data.split("-")[2];
    const revisioni = await getRevisione(page);
    const revisione = revisioni[0];

    if (revisione) {
      const photoUrl = revisione.attached;
      if (photoUrl) {
        await handleBotMessage(userMessages, chatId, "Allegato", {
          photo: photoUrl,
        });

        sendRevisione(chatId, userMessages, page);
      } else {
        console.error("No photo attached.");
      }
    }
  }

  if (msg.data.startsWith("con-rev-")) {
    const revisioneId = msg.data.split("-")[2];

    setStatusMission(userMessages, revisioneId, "complete");
    await handleBotMessage(
      userMessages,
      chatId,
      `✅ <b>Convalidata richiesta #${revisioneId}!</b>`,
      {
        parse_mode: "HTML",
      }
    );
    sendRevisione(chatId, userMessages, 1);
  }

  if (msg.data.startsWith("rif-rev-")) {
    const revisioneId = msg.data.split("-")[2];
    setStatusMission(userMessages, revisioneId, "rejected");
    await handleBotMessage(
      userMessages,
      chatId,
      `❌ <b>Rifiutata richiesta #${revisioneId}!</b>`,
      {
        parse_mode: "HTML",
        reply_markup: {
          inline_keyboard: [
            [
              {
                text: "➕ Aggiungi Motivo",
                callback_data: "mot-rev-" + revisioneId,
              },
            ],
            [
              {
                text: "➡️ Prossima Richiesta",
                callback_data: "rev-1",
              },
            ],
          ],
        },
      }
    );
  }

  if (msg.data.startsWith("rev-")) {
    const page = msg.data.split("-")[1];

    sendRevisione(chatId, userMessages, page);
  }

  if (msg.data.startsWith("m-")) {
    const [action, type, id] = msg.data.split("-");

    if (action === "m") {
      if (type === "appr") {
        await handleApproveMission(id);
        await displaySinglePendingMission(userMessages, chatId, 1);
      } else if (type === "delete") {
        await handleDeleteMission(id);
        await displaySinglePendingMission(userMessages, chatId, 1);
      } else if (type === "page") {
        const page = parseInt(id, 10);
        await displaySinglePendingMission(userMessages, chatId, page);
      }
    }
  }

  if (msg.data.startsWith("p-")) {
    const [action, type, id, userPay] = msg.data.split("-");

    const backKeyboard = [
      [{ text: "↩️ Torna indietro", callback_data: "annulla" }],
    ];

    if (action === "p") {
      if (type === "appr") {
        await handleBotMessage(
          userMessages,
          chatId,
          "Invia ora il codice di transazione o codice giftcard:",
          backKeyboard
        );

        paymentStats[chatId] = {
          type: "appr",
          transactionId: id,
          userPay: userPay,
        };
      } else if (type === "reject") {
        await handleBotMessage(
          userMessages,
          chatId,
          "Invia ora il motivo del rifiuto",
          backKeyboard
        );

        paymentStats[chatId] = {
          type: "reject",
          transactionId: id,
          userPay: userPay,
        };
      } else if (type === "page") {
        const page = parseInt(id, 10);
        await displaySinglePayment(userMessages, chatId, page);
        await bot.answerCallbackQuery(callbackQuery.id);
      }
    }
  }
};

module.exports = {
  handleAdmin,
  handleCallbackQuery,
};
