const TelegramBot = require("node-telegram-bot-api");
const mysql = require("mysql2");
const bot = new TelegramBot(process.env.TELEGRAM_TOKEN, { polling: false });
const stripe = require("stripe")(process.env.STRIPE_SECRET_KEY);
const moment = require("moment");

const db = mysql.createConnection({
  host: process.env.DB_HOST,
  user: process.env.DB_USER,
  password: process.env.DB_PASSWORD,
  database: process.env.DB_NAME,
});

const createMessage = (package) => {
  if (package === "1000 punti al costo di 4,99 € / fino a 200 interazioni") {
    return `Pacchetto Base da 1.000 Punti
1.000 punti al costo di 4,99 € 

❤️ fino a 200 interazioni
👥 Utenti Reali / No Bot 
👥 +18 anni
🇮🇹 100% Italiani`;
  }

  if (
    package === "5.000 punti al costo di 19,99 € / fino a 1000 interazioni -20%"
  ) {
    return `Pacchetto Premium da 5.000 Punti
5.000 punti al costo di 19,99 € 

❤️ Fino a 1000 interazioni
👥 Utenti Reali / No Bot 
👥 +18 anni
🇮🇹 100% Italiani
`;
  }

  if (package === "10 Follower 1,00 €") {
    return `Pacchetto 10 Follower
10 Follower al costo di 1,00 € 

👥 Utenti Reali / No Bot 
👥 +18 anni
🇮🇹 100% Italiani`;
  }
  if (package === "100 Follower 7,50 €") {
    return `Pacchetto 100 Follower
100 Follower al costo di 7,50 € 

👥 Utenti Reali / No Bot 
👥 +18 anni
🇮🇹 100% Italiani`;
  }
  if (package === "10 Commenti 2,00 €") {
    return `Pacchetto 10 Commenti
10 Commenti al costo di 2,00 € 

👥 Utenti Reali / No Bot 
👥 +18 anni
🇮🇹 100% Italiani`;
  }
  if (package === "20 Commenti 3,00 €") {
    return `Pacchetto 20 Commenti
20 Commenti al costo di 3,00 € 

👥 Utenti Reali / No Bot 
👥 +18 anni
🇮🇹 100% Italiani`;
  }
  if (package === "10 Iscrizioni a Gruppo 1,00 €") {
    return `Pacchetto 10 Iscrizioni
10 Iscrizioni al costo di 1,00 € 

👥 Utenti Reali / No Bot 
👥 +18 anni
🇮🇹 100% Italiani`;
  }
  if (package === "50 Iscrizioni a Gruppo 4,00 €") {
    return `Pacchetto 50 Iscrizioni
50 Iscrizioni al costo di 4,00 € 

👥 Utenti Reali / No Bot 
👥 +18 anni
🇮🇹 100% Italiani`;
  }
  if (package === "10 Commenti 1,00 €") {
    return `Pacchetto 10 Commenti
10 Commenti al costo di 1,00 € 

👥 Utenti Reali / No Bot 
👥 +18 anni
🇮🇹 100% Italiani`;
  }
  if (package === "20 Commenti 1,50 €") {
    return `Pacchetto 20 Commenti
20 Commenti al costo di 1,50 € 

👥 Utenti Reali / No Bot 
👥 +18 anni
🇮🇹 100% Italiani`;
  }
  if (package === "10 iscrizioni a Canale/Gruppo 1,00 €") {
    return `Pacchetto 10 iscrizioni a Canale/Gruppo
10 iscrizioni al costo di 1,00 € 

👥 Utenti Reali / No Bot 
👥 +18 anni
🇮🇹 100% Italiani`;
  }
  if (package === "50 Iscrizioni a Canale/Gruppo 2,50 €") {
    return `Pacchetto 50 iscrizioni a Canale/Gruppo
50 iscrizioni al costo di 2,50 € 

👥 Utenti Reali / No Bot 
👥 +18 anni
🇮🇹 100% Italiani`;
  }
};

const createStripeSession = async (userMessages, chatId, packageDetails) => {
  try {
    const session = await stripe.checkout.sessions.create({
      payment_method_types: ["card"],
      line_items: [
        {
          price_data: {
            currency: "eur",
            product_data: {
              name: packageDetails.name,
            },
            unit_amount: packageDetails.price, // in cents
          },
          quantity: 1,
        },
      ],
      mode: "payment",
      success_url: "https://example.com/success", // URL fittizio
      cancel_url: "https://example.com/cancel", // URL fittizio
      metadata: {
        package_id: packageDetails.name,
        telegram_id: chatId,
      },
    });

    await handleBotMessage(
      userMessages,
      chatId,
      createMessage(packageDetails.name),
      {
        parse_mode: "HTML",
        reply_markup: {
          inline_keyboard: [
            [
              {
                text: "Paga " + (packageDetails.price / 100).toFixed(2) + " €",
                url: session.url,
              },
            ],
            [{ text: "❌ Annulla", callback_data: "annulla" }],
          ],
        },
      }
    );
  } catch (err) {
    console.error("Stripe session creation failed:", err);
  }
};

const handleBotMessage = async (userMessages, chatId, text, options) => {
  try {
    const sentMessage = await bot.sendMessage(chatId, text, options);

    if (!userMessages[chatId]) {
      userMessages[chatId] = [];
    }

    userMessages[chatId].push(sentMessage.message_id);
  } catch (err) {
    console.error("Errore nell'invio del messaggio:", err);
  }
};

const userPayment = {};

const keyboardShop = {
  parse_mode: "HTML",
  reply_markup: {
    keyboard: [
      [{ text: "🛒 Acquista 🛒" }, { text: "📝 Acquisti Effettuati 📝" }],
      [{ text: "↩️ Torna indietro" }],
    ],
    resize_keyboard: true,
    one_time_keyboard: true,
  },
};

const handleShop = async (userMessages, chatId, userData, msg) => {
  if (msg.text === "/start" || msg.text === "↩️ Torna indietro") {
    userPayment[chatId] = null;
  }

  if (msg.text === "📝 Acquisti Effettuati 📝") {
    const query = `
        SELECT cp.*, p.description AS package_name, p.platform, m.count_user, m.max_user
        FROM completed_payments cp
        LEFT JOIN packages p ON cp.package_id = p.id
        LEFT JOIN missions m ON cp.mission_id = m.id
        WHERE cp.user_id = ?
    `;

    const userId = userData.id;

    db.execute(query, [userId], async (error, results) => {
      if (error) {
        console.error("Errore nel recupero degli acquisti:", error);
        handleBotMessage(
          userMessages,
          chatId,
          "Si è verificato un errore nel recupero degli acquisti.",
          keyboardShop
        );
      } else {
        if (results.length > 0) {
          results.forEach(async (payment) => {
            const formattedDate = moment(payment.created_at).format(
              "DD/MM/YYYY [ore] HH:mm"
            );

            let statusIndicator = payment.typePoint
              ? "🟢 Stato: Ricevuti"
              : payment.mission_id
              ? "🟢 Stato: Utilizzato"
              : "🟡 Stato: Da Utilizzare";

            const message = `
ID Acquisto: ${payment.id}
Pacchetto: ${payment.package_name ? payment.package_name : "Non specificato"}
Piattaforma: ${payment.platform}

${
  payment.count_user !== null
    ? `👥 ${payment.count_user} di ${payment.max_user} `
    : ""
}

Importo: ${(payment.amount / 100).toFixed(2)} €
Data: ${formattedDate}
  
${statusIndicator}
            `;

            if (!payment.typePoint && !payment.mission_id) {
              await handleBotMessage(userMessages, chatId, message, {
                parse_mode: "HTML",
                reply_markup: {
                  inline_keyboard: [
                    [
                      {
                        text: "🔐 Utilizza ora",
                        callback_data: "shop-payment-" + payment.id,
                      },
                    ],
                  ],
                },
              });
            } else {
              await handleBotMessage(
                userMessages,
                chatId,
                message,
                keyboardShop
              );
            }
          });
        } else {
          // Se non ci sono acquisti, informa l'utente
          await handleBotMessage(
            userMessages,
            chatId,
            "😔 Attualmente non hai effettuato nessun acquisto, continua a esplorare i pacchetti disponibili!",
            keyboardShop
          );
        }
      }
    });
  }

  if (msg.text === "🛒 SHOP 🛒") {
    const options = {
      parse_mode: "HTML",
      reply_markup: {
        keyboard: [
          [{ text: "🛒 Acquista 🛒" }, { text: "📝 Acquisti Effettuati 📝" }],
          [{ text: "↩️ Torna indietro" }],
        ],
        resize_keyboard: true,
        one_time_keyboard: true,
      },
    };
    await handleBotMessage(
      userMessages,
      chatId,
      `🛒 Benvenuto nello <b>Shop</b>

In questa sezione potrai acquistare o gestire gli acquisti.`,
      options
    );
  }

  if (msg.text === "🛒 Acquista 🛒") {
    const options = {
      parse_mode: "HTML",
      reply_markup: {
        keyboard: [
          [{ text: "🆙 Punti 🆙" }, { text: "📸 Instagram 📸" }],
          [{ text: "🟦 Facebook 🟦" }, { text: "💬 Telegram 💬" }],
          [{ text: "↩️ Torna indietro" }],
        ],
        resize_keyboard: true,
        one_time_keyboard: true,
      },
    };
    await handleBotMessage(
      userMessages,
      chatId,
      `<b>Vuoi raggiungere più velocemente i tuoi obiettivi?</b>
Acquista uno dei nostri pacchetti visibili in “Shop”.

Premi su uno dei menù sottostanti per ricevere il listino a te dedicato.`,
      options
    );
  }

  if (msg.text === "🆙 Punti 🆙") {
    const options = {
      reply_markup: {
        keyboard: [
          [
            { text: "1000 punti al costo di 4,99 € / fino a 200 interazioni" },
            {
              text: "5.000 punti al costo di 19,99 € / fino a 1000 interazioni -20%",
            },
          ],
          [{ text: "↩️ Torna indietro" }],
        ],
        resize_keyboard: true,
        one_time_keyboard: true,
      },
    };
    await handleBotMessage(
      userMessages,
      chatId,
      `Scegli il pacchetto di 🆙 Punti 🆙`,
      options
    );
  }

  if (msg.text === "1000 punti al costo di 4,99 € / fino a 200 interazioni") {
    const packageDetails = {
      name: "1000 punti al costo di 4,99 € / fino a 200 interazioni",
      price: 499,
    };
    await createStripeSession(userMessages, chatId, packageDetails);
  }

  if (
    msg.text ===
    "5.000 punti al costo di 19,99 € / fino a 1000 interazioni -20%"
  ) {
    const packageDetails = {
      name: "5.000 punti al costo di 19,99 € / fino a 1000 interazioni -20%",
      price: 1999,
    };
    await createStripeSession(userMessages, chatId, packageDetails);
  }

  if (msg.text === "📸 Instagram 📸") {
    const options = {
      reply_markup: {
        keyboard: [
          [
            { text: "10 Follower 1,00 €" },
            {
              text: "100 Follower 7,50 €",
            },
          ],
          [
            { text: "10 Commenti 2,00 €" },
            {
              text: "20 Commenti 3,00 €",
            },
          ],
          [{ text: "↩️ Torna indietro" }],
        ],
        resize_keyboard: true,
        one_time_keyboard: true,
      },
    };
    await handleBotMessage(
      userMessages,
      chatId,
      `Scegli il pacchetto di 📸 Instagram 📸`,
      options
    );
  }

  if (msg.text === "10 Follower 1,00 €") {
    const packageDetails = {
      name: "10 Follower 1,00 €",
      price: 100,
    };
    await createStripeSession(userMessages, chatId, packageDetails);
  }

  if (msg.text === "100 Follower 7,50 €") {
    const packageDetails = {
      name: "100 Follower 7,50 €",
      price: 750,
    };
    await createStripeSession(userMessages, chatId, packageDetails);
  }

  if (msg.text === "10 Commenti 2,00 €") {
    const packageDetails = {
      name: "10 Commenti 2,00 €",
      price: 200,
    };
    await createStripeSession(userMessages, chatId, packageDetails);
  }

  if (msg.text === "20 Commenti 3,00 €") {
    const packageDetails = {
      name: "20 Commenti 3,00 €",
      price: 300,
    };
    await createStripeSession(userMessages, chatId, packageDetails);
  }

  if (msg.text === "🟦 Facebook 🟦") {
    const options = {
      reply_markup: {
        keyboard: [
          [
            { text: "10 Iscrizioni a Gruppo 1,00 €" },
            {
              text: "50 Iscrizioni a Gruppo 4,00 €",
            },
          ],
          [
            { text: "10 Commenti 1,00 €" },
            {
              text: "20 Commenti 1,50 €",
            },
          ],
          [{ text: "↩️ Torna indietro" }],
        ],
        resize_keyboard: true,
        one_time_keyboard: true,
      },
    };
    await handleBotMessage(
      userMessages,
      chatId,
      `Scegli il pacchetto di 🟦 Facebook 🟦`,
      options
    );
  }

  if (msg.text === "10 Iscrizioni a Gruppo 1,00 €") {
    const packageDetails = {
      name: "10 Iscrizioni a Gruppo 1,00 €",
      price: 100,
    };
    await createStripeSession(userMessages, chatId, packageDetails);
  }

  if (msg.text === "50 Iscrizioni a Gruppo 4,00 €") {
    const packageDetails = {
      name: "50 Iscrizioni a Gruppo 4,00 €",
      price: 400,
    };
    await createStripeSession(userMessages, chatId, packageDetails);
  }

  if (msg.text === "10 Commenti 1,00 €") {
    const packageDetails = {
      name: "10 Commenti 1,00 €",
      price: 100,
    };
    await createStripeSession(userMessages, chatId, packageDetails);
  }

  if (msg.text === "20 Commenti 1,50 €") {
    const packageDetails = {
      name: "20 Commenti 1,50 €",
      price: 150,
    };
    await createStripeSession(userMessages, chatId, packageDetails);
  }

  if (msg.text === "💬 Telegram 💬") {
    const options = {
      reply_markup: {
        keyboard: [
          [
            { text: "10 iscrizioni a Canale/Gruppo 1,00 €" },
            {
              text: "50 Iscrizioni a Canale/Gruppo 2,50 €",
            },
          ],
          [{ text: "↩️ Torna indietro" }],
        ],
        resize_keyboard: true,
        one_time_keyboard: true,
      },
    };
    await handleBotMessage(
      userMessages,
      chatId,
      `Scegli il pacchetto di 💬 Telegram 💬`,
      options
    );
  }

  if (msg.text === "10 iscrizioni a Canale/Gruppo 1,00 €") {
    const packageDetails = {
      name: "10 iscrizioni a Canale/Gruppo 1,00 €",
      price: 100,
    };
    await createStripeSession(userMessages, chatId, packageDetails);
  }

  if (msg.text === "50 Iscrizioni a Canale/Gruppo 2,50 €") {
    const packageDetails = {
      name: "50 Iscrizioni a Canale/Gruppo 2,50 €",
      price: 250,
    };
    await createStripeSession(userMessages, chatId, packageDetails);
  }

  if (userPayment[chatId]) {
    const paymentId = userPayment[chatId];

    const [rows] = await db
      .promise()
      .query(
        "SELECT *, p.platform, p.action_type, p.description FROM completed_payments as cp LEFT JOIN packages p ON cp.package_id = p.id WHERE cp.id = ? AND cp.user_id = ? AND cp.mission_id IS NULL",
        [paymentId, userData.id]
      );

    if (rows.length > 0) {
      const newMissionId = await createMissionFromPackage(
        msg.text,
        rows[0].platform,
        rows[0].action_type,
        rows[0].max_user,
        userData,
        chatId
      );
      await db
        .promise()
        .query("UPDATE completed_payments SET mission_id = ? WHERE id = ?", [
          newMissionId,
          paymentId,
        ]);
      await handleBotMessage(
        userMessages,
        chatId,
        "Pacchetto utilizzato con successo",
        keyboardShop
      );
      userPayment[chatId] = null;
    } else {
      await handleBotMessage(userMessages, chatId, "Pacchetto già utilizzato.");
    }
  }
};

const createMissionFromPackage = async (
  link,
  platform,
  action_type,
  max_user,
  userData,
  chatId
) => {
  const instructions = `${action_type} su ${platform} al seguente link`;

  const query = `
  INSERT INTO missions (type, platform, points, instructions, link, max_user, count_user, status, user_admin, user_id)
  VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?)`;

  try {
    const [result] = await db
      .promise()
      .query(query, [
        "close_number",
        null,
        20,
        instructions,
        link,
        max_user,
        0,
        "active",
        1,
        userData.id,
      ]);

    const missionId = result.insertId; // Ottenere l'ID della missione inserita

    return missionId; // Restituisci l'ID della missione
  } catch (err) {
    console.error("Errore nell'inserimento della missione:", err);
    await bot.sendMessage(
      chatId,
      "Si è verificato un errore durante la creazione della missione."
    );
    return null; // Restituisci null in caso di errore
  }
};

const handleCallbackQuery = async (userMessages, chatId, userData, msg) => {
  if (msg.data.startsWith("shop-payment-")) {
    const paymentId = msg.data.replace("shop-payment-", "");
    userPayment[chatId] = paymentId;
    const options = {
      reply_markup: {
        keyboard: [[{ text: "↩️ Torna indietro" }]],
        resize_keyboard: true,
      },
    };
    await handleBotMessage(
      userMessages,
      chatId,
      `Digita ora il link da utilizzare per la missione acquistata:`,
      options
    );
  }
};

module.exports = {
  handleShop,
  handleCallbackQuery,
};
