const TelegramBot = require("node-telegram-bot-api");
const mysql = require("mysql2");
const bot = new TelegramBot(process.env.TELEGRAM_TOKEN, { polling: false });
const { v4: uuidv4 } = require("uuid");
const moment = require("moment");

function generateUUID() {
  return uuidv4(); // Ritorna un UUID v4
}

const db = mysql.createConnection({
  host: process.env.DB_HOST,
  user: process.env.DB_USER,
  password: process.env.DB_PASSWORD,
  database: process.env.DB_NAME,
});

const handleBotMessage = async (userMessages, chatId, text, options) => {
  try {
    const sentMessage = await bot.sendMessage(chatId, text, options);

    if (!userMessages[chatId]) {
      userMessages[chatId] = [];
    }

    userMessages[chatId].push(sentMessage.message_id);
  } catch (err) {
    console.error("Errore nell'invio del messaggio:", err);
  }
};

const userConvert = {};

const keyboardConvert = [
  [
    { text: "♻️ Converti Portafoglio Missioni ♻️" },
    { text: "♻️ Converti Portafoglio Amici ♻️" },
  ],
  [{ text: "↩️ Torna indietro" }],
];

const userPay = {};

const keyboardPrelievo = {
  parse_mode: "HTML",
  reply_markup: {
    keyboard: [
      [
        { text: "📤 Richiedi un Prelievo 📤" },
        { text: "🗒 Prelievi Effettuati 🗒" },
      ],
      [{ text: "↩️ Torna indietro" }],
    ],
    resize_keyboard: true,
  },
};

const checkWallet = async (userMessages, chatId, userData) => {
  if (userData.wallet < 5) {
    await handleBotMessage(
      userMessages,
      chatId,
      `Ci dispiace! Non hai ancora raggiunto il minimo importo prelevabile. Continua ad accumulare punti.`,
      keyboardPrelievo
    );
    return false;
  } else {
    return true;
  }
};

const handleWallet = async (userMessages, chatId, userData, msg) => {
  if (msg.text === "/start" || msg.text === "↩️ Torna indietro") {
    userConvert[chatId] = null;
    userPay[chatId] = null;
  }

  if (msg.text === "🗒 Prelievi Effettuati 🗒") {
    const query = `
        SELECT *
        FROM transactions
        WHERE userId = ?
    `;

    const userId = userData.id;

    db.execute(query, [userId], async (error, results) => {
      if (error) {
        console.error("Errore nel recupero delle transazioni:", error);
        bot.sendMessage(
          chatId,
          "Si è verificato un errore nel recupero delle transazioni."
        );
      } else {
        if (results.length > 0) {
          let totalP = 0;

          results.forEach(async (transaction) => {
            const formattedDate = moment(transaction.created_at).format(
              "DD/MM/YYYY [ore] HH:mm"
            );

            let statusIndicator;
            switch (transaction.status) {
              case "paid":
                statusIndicator = "🟢 Stato: Pagato";
                break;
              case "in_progress":
                statusIndicator = "🟡 Stato: In lavorazione";
                break;
              case "cancelled":
                statusIndicator = "🔴 Stato: Annullato";
                break;
              default:
                statusIndicator = "Stato sconosciuto";
            }

            const message = `
ID Richiesta: ${transaction.uuid}
Metodo: ${transaction.type}
${transaction.wallet ? "Indirizzo: " + transaction.wallet : ""}
${
  transaction.transaction_code
    ? transaction.type === "Crypto"
      ? "Numero transazione: " + transaction.transaction_code
      : "Codice: " + transaction.transaction_code
    : ""
}
Importo: ${transaction.amount} €
Data: ${formattedDate}

${statusIndicator}
${transaction.reason ? "Motivo: " + transaction.reason : ""}
                    `;

            totalP = parseFloat(totalP) + parseFloat(transaction.amount);
            await handleBotMessage(userMessages, chatId, message);
          });

          await handleBotMessage(
            userMessages,
            chatId,
            `🥳 Complimenti! Ad oggi hai prelevato ${totalP} €`,
            keyboardPrelievo
          );
        } else {
          // Se non ci sono transazioni, informa l'utente
          handleBotMessage(
            userMessages,
            chatId,
            "😔 Attualmente non hai effettuato nessun prelievo, continua a racimulare punti!",
            keyboardPrelievo
          );
        }
      }
    });
  }

  if (msg.text === "💼 WALLET") {
    const wallet = parseFloat(userData.wallet_missions) || 0;
    const walletBuy = parseFloat(userData.wallet_buy) || 0;
    const walletFriends = parseFloat(userData.wallet_friends) || 0;

    const totaleWallet = wallet + walletBuy + walletFriends;

    const convert = (wallet + walletFriends) * 0.001;

    await handleBotMessage(
      userMessages,
      chatId,
      `<b>Guarda il tuo saldo punti su SocialLife!</b> 🎉
Pronto a vedere quanto hai guadagnato?
  
🤖 <b>Status:</b> ${userData.role}
  
🅿️ <b>Saldo punti:</b> ${totaleWallet.toFixed(2)}
  
➡️ <b>Punti acquistati:</b> ${walletBuy.toFixed(2)} (non convertibili in denaro)
  
➡️ <b>Punti guadagnati dalle Missioni:</b> ${wallet.toFixed(2)}
  
➡️ <b>Punti guadagnati dalla Rete di Amici:</b> ${walletFriends.toFixed(
        2
      )} (20% dei loro punti)
  
💶 <b>Stima dei tuoi guadagni:</b> ${convert} €
💰 <b>Denaro prelevabile:</b> ${userData.wallet} €
  
❗️ L’importo minimo per il prelievo è di <b>5,00 €</b> `,
      {
        parse_mode: "HTML",
        reply_markup: {
          keyboard: [
            [{ text: "♻️ Converti i punti ♻️" }, { text: "💲 Prelievo 💲" }],
            [{ text: "↩️ Torna indietro" }],
          ],
          resize_keyboard: true,
        },
      }
    );
  }

  if (msg.text === "💲 Prelievo 💲") {
    await handleBotMessage(
      userMessages,
      chatId,
      `NB. Il minimo payout è di 5.00 €`,
      keyboardPrelievo
    );
  }

  if (msg.text === "📤 Richiedi un Prelievo 📤") {
    if (await checkWallet(userMessages, chatId, userData)) {
      await handleBotMessage(
        userMessages,
        chatId,
        `Come preferisci essere pagato?`,
        {
          parse_mode: "HTML",
          reply_markup: {
            keyboard: [
              [{ text: "💱 Crypto Valute💱" }, { text: "💳 Gift Card 💳" }],
              [{ text: "↩️ Torna indietro" }],
            ],
            resize_keyboard: true,
          },
        }
      );
    }
  }

  if (userPay[chatId]) {
    if (userPay[chatId].method === "crypto") {
      if (userPay[chatId].wallet) {
        if (userPay[chatId].amount) {
          if (msg.text === "✅ Conferma") {
            const wallet = parseFloat(userData.wallet);

            const newBalance = wallet - parseFloat(userPay[chatId].amount);

            const updateQuery = `UPDATE telegram_users SET wallet = ? WHERE telegram_id = ?`;
            await db
              .promise()
              .query(updateQuery, [newBalance, userData.telegram_id]);

            const query = `
                                INSERT INTO transactions (userId, uuid, type, amount, status, wallet, reason)
                                VALUES (?, ?, 'Crypto', ?, 'in_progress', ?, ?)
                            `;

            const values = [
              userData.id,
              generateUUID(),
              userPay[chatId].amount,
              userPay[chatId].wallet,
              null,
            ];

            db.execute(query, values, async (error, results) => {
              if (error) {
                console.error(
                  "Errore nell'inserimento della transazione:",
                  error
                );
              } else {
                await handleBotMessage(
                  userMessages,
                  chatId,
                  `Prelievo in attesa di approvazione... visualizzalo in prelievi effettuati!`,
                  keyboardPrelievo
                );
              }
            });
          }
        } else {
          const amount = parseFloat(msg.text);

          if (amount > userData.wallet) {
            await handleBotMessage(
              userMessages,
              chatId,
              `Attenzione, inserisci un valore che possiedi nel portafoglio convertito!`,
              {
                parse_mode: "HTML",
                reply_markup: {
                  keyboard: [[{ text: "↩️ Torna indietro" }]],
                  resize_keyboard: true,
                },
              }
            );
          } else {
            userPay[chatId].amount = amount;

            await handleBotMessage(
              userMessages,
              chatId,
              `Importo Prelievo: ${amount}\nIndirizzo: ${userPay[chatId].wallet}\n\nUna volta confermato l'operazione è irreversibile!`,
              {
                parse_mode: "HTML",
                reply_markup: {
                  keyboard: [
                    [{ text: "✅ Conferma" }],
                    [{ text: "↩️ Torna indietro" }],
                  ],
                  resize_keyboard: true,
                },
              }
            );
          }
        }
      } else {
        userPay[chatId].wallet = msg.text;
        await handleBotMessage(
          userMessages,
          chatId,
          `Indirizzo: ${userPay[chatId].wallet}\n\nInvia ora quanto desideri prelevare`,
          {
            parse_mode: "HTML",
            reply_markup: {
              keyboard: [[{ text: "↩️ Torna indietro" }]],
              resize_keyboard: true,
            },
          }
        );
      }
    }

    if (userPay[chatId].method === "giftcard") {
      if (userPay[chatId].typeCard) {
        if (userPay[chatId].amount) {
          if (msg.text === "✅ Conferma") {
            const wallet = parseFloat(userData.wallet);

            const newBalance = wallet - parseFloat(userPay[chatId].amount);

            const updateQuery = `UPDATE telegram_users SET wallet = ? WHERE telegram_id = ?`;
            await db
              .promise()
              .query(updateQuery, [newBalance, userData.telegram_id]);

            const query = `
                                INSERT INTO transactions (userId, uuid, type, type_gift, amount, status)
                                VALUES (?, ?, 'GiftCard', ?, ?, 'in_progress')
                            `;

            const values = [
              userData.id,
              generateUUID(),
              userPay[chatId].typeCard,
              userPay[chatId].amount,
            ];

            db.execute(query, values, async (error, results) => {
              if (error) {
                console.error(
                  "Errore nell'inserimento della transazione:",
                  error
                );
              } else {
                await handleBotMessage(
                  userMessages,
                  chatId,
                  `Prelievo in attesa di approvazione... visualizzalo in prelievi effettuati!`,
                  keyboardPrelievo
                );
              }
            });
          }
        } else {
          const amount = parseFloat(msg.text.split(" ")[3]);

          if (amount > userData.wallet) {
            await handleBotMessage(
              userMessages,
              chatId,
              `Attenzione, inserisci un valore che possiedi nel portafoglio convertito!`,
              {
                parse_mode: "HTML",
                reply_markup: {
                  keyboard: [[{ text: "↩️ Torna indietro" }]],
                  resize_keyboard: true,
                },
              }
            );
          } else {
            userPay[chatId].amount = amount;

            await handleBotMessage(
              userMessages,
              chatId,
              `Importo Prelievo: ${amount}\nTipo GiftCard: ${userPay[chatId].typeCard}\n\nUna volta confermato l'operazione è irreversibile!`,
              {
                parse_mode: "HTML",
                reply_markup: {
                  keyboard: [
                    [{ text: "✅ Conferma" }],
                    [{ text: "↩️ Torna indietro" }],
                  ],
                  resize_keyboard: true,
                },
              }
            );
          }
        }
      } else {
        await handleBotMessage(
          userMessages,
          chatId,
          `Seleziona il valore della tua Gift Card ${msg.text}`,
          {
            parse_mode: "HTML",
            reply_markup: {
              keyboard: [
                [{ text: "Gift Card da 5.00 €" }],
                [{ text: "Gift Card da 10.00 €" }],
                [{ text: "Gift Card da 25.00 €" }],
                [{ text: "↩️ Torna indietro" }],
              ],
              resize_keyboard: true,
            },
          }
        );

        userPay[chatId] = { method: "giftcard", typeCard: msg.text };
      }
    }
  }

  if (msg.text === "💳 Gift Card 💳") {
    if (await checkWallet(userMessages, chatId, userData)) {
      await handleBotMessage(
        userMessages,
        chatId,
        `Seleziona la tua Gift Card`,
        {
          parse_mode: "HTML",
          reply_markup: {
            keyboard: [
              [{ text: "AMAZON" }, { text: "ZALANDO" }],
              [{ text: "EUROSPIN" }, { text: "APPLE" }],
              [{ text: "ETC ETC" }, { text: "ETC" }],
              [{ text: "↩️ Torna indietro" }],
            ],
            resize_keyboard: true,
          },
        }
      );

      userPay[chatId] = { method: "giftcard" };
    }
  }

  if (msg.text === "💱 Crypto Valute💱") {
    if (await checkWallet(userMessages, chatId, userData)) {
      await handleBotMessage(
        userMessages,
        chatId,
        `Inserisci l’indirizzo del tuo Wallet USDT.
NB. Verrà accreditato l’importo meno la Tassa di prelievo.`,
        {
          parse_mode: "HTML",
          reply_markup: {
            keyboard: [[{ text: "↩️ Torna indietro" }]],
            resize_keyboard: true,
          },
        }
      );
      userPay[chatId] = { method: "crypto" };
    }
  }

  if (msg.text === "♻️ Converti i punti ♻️") {
    await handleBotMessage(
      userMessages,
      chatId,
      "Scegli da dove prendere i punti da convertire",
      {
        parse_mode: "HTML",
        reply_markup: {
          keyboard: keyboardConvert,
          resize_keyboard: true,
        },
      }
    );
  }

  if (userConvert[chatId] && userConvert[chatId].startsWith("wallet_")) {
    const userField = userConvert[chatId];

    const value = parseInt(msg.text, 10);
    if (isNaN(value) || value <= 0) {
      await handleBotMessage(
        userMessages,
        chatId,
        "❌ Inserisci un numero intero valido maggiore di 0."
      );
      return;
    }

    const balance = parseInt(userData[userField], 10);

    if (balance >= value) {
      const amountToAdd = value * 0.001;

      const wallet = parseFloat(userData.wallet);
      const newBalance = wallet + amountToAdd;

      const updateQuery = `UPDATE telegram_users SET wallet = ? , ${userField} = ? WHERE telegram_id = ?`;
      await db
        .promise()
        .query(updateQuery, [
          newBalance,
          balance - value,
          userData.telegram_id,
        ]);

      await handleBotMessage(
        userMessages,
        chatId,
        `✅ Aggiornamento riuscito! Il tuo nuovo saldo è: ${newBalance.toFixed(
          3
        )}`,
        {
          parse_mode: "HTML",
          reply_markup: {
            keyboard: [
              [{ text: "♻️ Converti i punti ♻️" }, { text: "💲 Prelievo 💲" }],
              [{ text: "↩️ Torna indietro" }],
            ],
            resize_keyboard: true,
          },
        }
      );
      userConvert[chatId] = null;
    } else {
      await handleBotMessage(
        userMessages,
        chatId,
        `❌ Non hai abbastanza fondi per effettuare questa operazione.`,
        {
          parse_mode: "HTML",
          reply_markup: {
            keyboard: keyboardConvert,
            resize_keyboard: true,
          },
        }
      );
      userConvert[chatId] = null;
    }
  }

  if (msg.text === "♻️ Converti Portafoglio Missioni ♻️") {
    await handleBotMessage(
      userMessages,
      chatId,
      `♻️ Quanti punti desideri convertire??\n\n<i>Attualmente possiedi ${userData.wallet_missions} Punti</i>`,
      {
        parse_mode: "HTML",
        reply_markup: {
          keyboard: [[{ text: "↩️ Torna indietro" }]],
          resize_keyboard: true,
        },
      }
    );

    userConvert[chatId] = "wallet_missions";
  }

  if (msg.text === "♻️ Converti Portafoglio Amici ♻️") {
    await handleBotMessage(
      userMessages,
      chatId,
      `♻️ Quanti punti desideri convertire??\n\n<i>Attualmente possiedi ${userData.wallet_friends} Punti</i>`,
      {
        parse_mode: "HTML",
        reply_markup: {
          keyboard: [[{ text: "↩️ Torna indietro" }]],
          resize_keyboard: true,
        },
      }
    );

    userConvert[chatId] = "wallet_friends";
  }
};

module.exports = {
  handleWallet,
};
