require("dotenv").config();
const cron = require("node-cron");
const TelegramBot = require("node-telegram-bot-api");
const mysql = require("mysql2");

// Connessione al database
const db = mysql.createConnection({
  host: process.env.DB_HOST,
  user: process.env.DB_USER,
  password: process.env.DB_PASSWORD,
  database: process.env.DB_NAME,
});

// Connessione al bot Telegram
const bot = new TelegramBot(process.env.TELEGRAM_TOKEN, { polling: false });

const userMessages = {}; // Oggetto per tracciare l'ID dell'ultimo messaggio inviato per ciascun utente

// Funzione per inviare e gestire i messaggi
const sendScheduledMessage = async (chatId, text) => {
  // Cancella l'ultimo messaggio inviato, se presente
  console.log(userMessages[chatId]);
  if (userMessages[chatId]) {
    try {
      await bot.deleteMessage(chatId, userMessages[chatId]);
    } catch (err) {
      console.error("Errore nell'eliminazione del messaggio:", err);
    }
  }

  // Invia il nuovo messaggio
  try {
    // {
    //   reply_markup: {
    //     inline_keyboard: [[{ text: "🤑 GUADAGNA 🤑", callback_data: "earn" }]],
    //   },
    // }
    const sentMessage = await bot.sendMessage(chatId, text);

    // Salva l'ID dell'ultimo messaggio inviato
    userMessages[chatId] = sentMessage.message_id;
  } catch (err) {
    console.error("Errore nell'invio del messaggio:", err);
  }
};

// Funzione per ottenere i telegram_id degli utenti dal database
const getTelegramIds = () => {
  return new Promise((resolve, reject) => {
    db.query("SELECT telegram_id FROM telegram_users", (error, results) => {
      if (error) {
        return reject(error);
      }
      const ids = results.map((row) => row.telegram_id);
      resolve(ids);
    });
  });
};

// Funzione per impostare i cron job per tutti gli utenti
const setupCronJobs = async () => {
  const messageText =
    "👀 ⛔️NON È ANCORA FINITA!!⛔️\n" +
    "Ci sono ancora 🤩 tantissime campagne che ti aspettano!\n" +
    "Non perdere altro tempo e INIZIA ORA a 💰 guadagnare in modo facile e divertente.\n" +
    "Clicca subito su 🤑 GUADAGNA 🤑";

  // Ottieni tutti i telegram_id dal database
  const telegramIds = await getTelegramIds();

  // Programma i messaggi per ogni utente
  telegramIds.forEach((chatId) => {
    cron.schedule(
      "0 8 * * *",
      () => sendScheduledMessage(chatId, messageText),
      { timezone: "Europe/Rome" }
    );
    cron.schedule(
      "0 13 * * *",
      () => sendScheduledMessage(chatId, messageText),
      { timezone: "Europe/Rome" }
    );
    cron.schedule(
      "0 15 * * *",
      () => sendScheduledMessage(chatId, messageText),
      { timezone: "Europe/Rome" }
    );
    cron.schedule(
      "0 19 * * *",
      () => sendScheduledMessage(chatId, messageText),
      { timezone: "Europe/Rome" }
    );
    cron.schedule(
      "0 22 * * *",
      () => sendScheduledMessage(chatId, messageText),
      { timezone: "Europe/Rome" }
    );
    cron.schedule(
      "0 0 * * *",
      () => sendScheduledMessage(chatId, messageText),
      { timezone: "Europe/Rome" }
    );
  });
};

// Esegui la funzione per impostare i cron job
setupCronJobs().catch((err) =>
  console.error("Errore nell'impostare i cron job:", err)
);

module.exports = {
  setupCronJobs,
};
