require("dotenv").config();
const TelegramBot = require("node-telegram-bot-api");
const stripe = require("stripe")(process.env.STRIPE_SECRET_KEY);
const express = require("express");
const bodyParser = require("body-parser"); // Per gestire il parsing del body
const app = express();
const endpointSecret = process.env.STRIPE_WEBHOOK_SECRET;
const mysql = require("mysql2");

const bot = new TelegramBot(process.env.TELEGRAM_TOKEN, { polling: false });

const db = mysql.createConnection({
  host: process.env.DB_HOST,
  user: process.env.DB_USER,
  password: process.env.DB_PASSWORD,
  database: process.env.DB_NAME,
});

const dbPromise = db.promise();

const packageManagment = async (package, userData) => {
  let pointsAdded;

  if (package === "1000 punti al costo di 4,99 € / fino a 200 interazioni") {
    pointsAdded = parseFloat(userData.wallet_buy) + 1000;
  }

  if (
    package === "5.000 punti al costo di 19,99 € / fino a 1000 interazioni -20%"
  ) {
    pointsAdded = parseFloat(userData.wallet_buy) + 5000;
  }

  if (pointsAdded !== undefined) {
    try {
      const query = `
        UPDATE telegram_users 
        SET wallet_buy = ? 
        WHERE telegram_id = ?`;

      await dbPromise.execute(query, [pointsAdded, userData.telegram_id]);
      bot.sendMessage(
        userData.telegram_id,
        "🥳 Complimenti! Il tuo portafoglio punti è stato aggiornato a " +
          pointsAdded +
          " Punti"
      );
    } catch (error) {
      console.error("Error updating wallet_buy:", error);
    }
  }
};

// Middleware per il parsing del body
app.use(bodyParser.raw({ type: "application/json" }));

// Ricevi eventi webhook
app.post("/webhook", async (req, res) => {
  const sig = req.headers["stripe-signature"];

  let event;
  try {
    event = stripe.webhooks.constructEvent(req.body, sig, endpointSecret);
  } catch (err) {
    console.error(`Webhook Error: ${err.message}`);
    res.status(400).send(`Webhook Error: ${err.message}`);
    return;
  }

  // Gestisci l'evento di completamento del checkout
  if (event.type === "checkout.session.completed") {
    const session = event.data.object;
    const packageDescription = session.metadata.package_id; // Assicurati che questo corrisponda ai dati nella metadata
    const telegramId = session.metadata.telegram_id;

    let typePoint = false;
    if (
      packageDescription ===
        "1000 punti al costo di 4,99 € / fino a 200 interazioni" ||
      packageDescription ===
        "5.000 punti al costo di 19,99 € / fino a 1000 interazioni -20%"
    ) {
      typePoint = true;
    }

    try {
      // 1. Ottieni l'ID del pacchetto dalla descrizione
      const [packageRows] = await dbPromise.query(
        "SELECT id FROM packages WHERE description = ?",
        [packageDescription]
      );

      if (packageRows.length === 0) {
        throw new Error(
          `Pacchetto non trovato per la descrizione: ${packageDescription}`
        );
      }

      const packageId = packageRows[0].id;

      // 2. Ottieni l'ID dell'utente da telegramId
      const [userRows] = await dbPromise.query(
        "SELECT * FROM telegram_users WHERE telegram_id = ?",
        [telegramId]
      );

      if (userRows.length === 0) {
        throw new Error(`Utente Telegram non trovato per l'ID: ${telegramId}`);
      }

      const userId = userRows[0].id;

      // 3. Inserisci il pagamento completato nella tabella completed_payments
      await dbPromise.query(
        "INSERT INTO completed_payments (package_id, user_id, amount,typePoint) VALUES (?, ?, ?, ?)",
        [packageId, userId, session.amount_total, typePoint]
      );

      const options = {
        parse_mode: "HTML",
        reply_markup: {
          keyboard: [
            [{ text: "🛒 Acquista 🛒" }, { text: "📝 Acquisti Effettuati 📝" }],
            [{ text: "↩️ Torna indietro" }],
          ],
          resize_keyboard: true,
          one_time_keyboard: true,
        },
      };

      bot.sendMessage(
        telegramId,
        `🥳 Complimenti! L'acquisto del pacchetto è stato completato con successo! ${
          !typePoint ? "\nInserisci il link in acquisti effettuati" : ""
        }`,
        options
      );

      packageManagment(packageDescription, userRows[0]);

      res.status(200).send("Pagamento completato e registrato con successo.");
    } catch (err) {
      console.error(`Errore durante la gestione del pagamento: ${err.message}`);
      res
        .status(500)
        .send(`Errore durante la gestione del pagamento: ${err.message}`);
    }
  }

  res.status(200).end();
});

app.listen(5000, () => {
  console.log("Server in ascolto sulla porta 5000");
});
